(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2008,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* equations *)

open Term;;

type eqn = term * term;;

(* equality up to renaming *)

open Unif;;
open Subs;;
open Rename;;

let compare ((l1,r1) as e1) ((l2,r2) as e2) =
  let rho = renaming_aways [l1; r1] in
  let l2 = rename rho l2 and r2 = rename rho r2 in
    try
      let _ = inverse (unif [l1,l2; r1,r2]) in 0
    with Failure _ -> Pervasives.compare e1 e2;;

(* print an equation *)

let bprint b (l,r) =
  Printf.bprintf b "%a = %a" Term.bprint l Term.bprint r;;

(* useful modules *)

module OrdEqn = struct
  type t = eqn;;
  let compare = compare;;
  let bprint = bprint;;
end;;

module EqnSet = Myset.Make (OrdEqn);;

let add_non_trivial f (l,r) es =
  let l = f l and r = f r in
    if l = r then es else EqnSet.add (l,r) es;;

let remove_trivial f es = EqnSet.fold (add_non_trivial f) es EqnSet.empty;;

(* test if an equation is commutativity *)

let is_commutativity = function
  | App (f1, [Var x1; Var y1]), App (f2, [Var y2; Var x2]) ->
      f1 = f2 && x1 = x2 && y1 = y2
  | _ -> false;;

let has_commutativity = EqnSet.exists is_commutativity;;

let remove_commutativity = EqnSet.filter (fun e -> not (is_commutativity e));;
