(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2008,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* rewrite rules. the variables of the rhs must be included in the lhs *)

open Term;;

type rule = term * term;;

(* equality up to lhs-variables renaming *)

open Match;;
open Subs;;
open Rename;;

let compare ((l1,r1) as e1) ((l2,r2) as e2) =
  match eq_opt l1 l2 with
    | Some s1 -> Pervasives.compare (apply s1 r1) r2
    | None -> Pervasives.compare e1 e2;;

(* print a rule *)

let bprint b (l,r) = Printf.bprintf b "%a -> %a" Term.bprint l Term.bprint r;;

(* useful modules *)

module OrdRul = struct
  type t = rule;;
  let compare = compare;;
  let bprint = bprint;;
end;;

module RulSet = Myset.Make (OrdRul);;

(* symbols of a set of rules *)

open Symb;;

let symbols_of_rule (l, r) = SymbSet.union (symbols l) (symbols r);;

let symbols_of_rules =
  let f r s = SymbSet.union (symbols_of_rule r) s in
    fun rs -> RulSet.fold f rs SymbSet.empty
;;
