(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frédéric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2007,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* $Id: t_otype.ml,v 1.1 2007/12/03 14:19:02 lowenthl Exp $ *)

(** {Functions on OCaml types.} *)

open Parsetree

(** Syntactic comparison ignoring locations *)
let rec eq_type t1 t2 = eq_type_desc t1.ptyp_desc t2.ptyp_desc

and eq_type_desc td1 td2 = match td1, td2 with
    Ptyp_any, Ptyp_any -> true
  | Ptyp_var v1, Ptyp_var v2 ->
     v1 = v2
  | Ptyp_arrow (l1, ct_arg1, ct_res1), Ptyp_arrow (l2, ct_arg2, ct_res2) ->
     l1 = l2 &&
     eq_type ct_arg1 ct_arg2 &&
     eq_type ct_res1 ct_res2
  | Ptyp_tuple cts1, Ptyp_tuple cts2 ->
     eq_types cts1 cts2
  | Ptyp_constr (id1, cts1), Ptyp_constr (id2, cts2) ->
     id1 = id2 &&
     eq_types cts1 cts2
  | Ptyp_object cfts1, Ptyp_object cfts2 ->
     eq_fields cfts1 cfts2
  | Ptyp_class (id1, cts1, lbs1), Ptyp_class (id2, cts2, lbs2) ->
     id1 = id2 &&
     eq_types cts1 cts2 &&
     lbs1 = lbs2
  | Ptyp_alias (ct1, s1), Ptyp_alias (ct2, s2) ->
     ct1 = ct2 &&
     s1 = s2
  | Ptyp_variant (rfs1, b1, lbs1), Ptyp_variant (rfs2, b2, lbs2) -> 
     eq_row_fields rfs1 rfs2 &&
     b1 = b2 &&
     lbs1 = lbs2
  | Ptyp_poly (ss1, ct1), Ptyp_poly (ss2, ct2) ->
     ss1 = ss2 &&
     eq_type ct1 ct2
  | Ptyp_parens ct1, Ptyp_parens ct2 ->
     eq_type ct1 ct2
  | _, _ -> false

and eq_types cts1 cts2 =
  try List.for_all2 eq_type cts1 cts2
  with Invalid_argument _ -> false

and eq_field cft1 cft2 = eq_field_desc cft1.pfield_desc cft2.pfield_desc

and eq_field_desc cfd1 cfd2 = match cfd1, cfd2 with
    Pfield (s1, ct1), Pfield (s2, ct2) ->
      s1 = s2 &&
      eq_type ct1 ct2
  | Pfield_var, Pfield_var -> true
  | Pfield_var, Pfield _
  | Pfield _, Pfield_var -> false

and eq_fields cfs1 cfs2 =
  try List.for_all2 eq_field cfs1 cfs2
  with Invalid_argument _ -> false

and eq_row_field rf1 rf2 = match rf1, rf2 with
    Rtag (l1, b1, cts1), Rtag (l2, b2, cts2) ->
      l1 = l2 &&
      b1 = b2 &&
      eq_types cts1 cts2
  | Rinherit ct1, Rinherit ct2 ->
      eq_type ct1 ct2
  | Rtag _, Rinherit _
  | Rinherit _, Rtag _ -> false

and eq_row_fields rfs1 rfs2 =
  try List.for_all2 eq_row_field rfs1 rfs2
  with Invalid_argument _ -> false
;;

